% This code replicates the main results of the paper "Fiscal stimulus in
% times of high debt: reconsidering multipliers and twin deficits" by
% Christiane Nickel and Andreas Tudyka.

% Note that the code heavily draws from Towbin and Weber's (2011) code
% who kindly made their code available.

% Please refer to the README file for further details

% %==========================================================================
% The following countries are in the dataset - each with the same number of
% time observations!!!: % above 70% debt/gdp at some point in time
%--------------------------------------
% 'AUT'	'BEL'	'BGR'	'CYP'	'DEU'	'DNK'	'FRA'	'GBR'	'GRC'	'HUN'	'IRL'	'ISL'	'ITA'	'MLT'	'NLD'	'PRT'	'SWE'
%=========================================================================

%%%%%%%%%%%%%%%%
clear all
load data
load countrynames.mat

tic % start timer

rgovcons = data(:,1); % real government consumption
realgdp = data(:,2); % real GDP
rprivinv = data(:,3); % real private investment
debtratio = data(:,4); % debt ratio
netexp = data(:,5); % net exports
ltrate = data(:,6); % real long term interest rate
gdpdefl = data(:,7); % GDP deflator

%%%%%%%%%%%%%%
%------------------------------------
% Define Matrix of Interaction Terms
%------------------------------------
Idata = log(debtratio);
origIdata=debtratio;
%------------------------------------
% Figure of interaction variable distribution
%------------------------------------
fig_inter_distr
% %------------------------------------
% Setting Estimation Properties
%------------------------------------

lag=1;        % Number of lags
nexog=0;        % Number of exogenous variables/if 1 no interaction terms in 1st eq., if 2 no int terms in eq. 1 and 2 and so on
I=size(countrynames,1);           % Number of cross sectional units (countries)
period=21;      % Length of IRFs
demean=2;       % 0 = fixed effects/demean, 1 = linear detrending, 2 = quadratic detrending
timing=-1;       % 0 = interaction always with time t value; 1 = always respective lag; -i = with t_(-i) value - must be <=lag
shock=1;        % Shock to variable number 'shock' will be plotted in IR graphs
percentgdpIRF = 1; % If 1 IRF to a 1% of GDP shock to G of Y, I etc. in terms of % of GDP, if the shock and responses are in percentage points.

nrep = 1000;     % # of kept draws
nburn = 2000;    % # of burn-in draws

imppics = 1;    % make graphs of impulse responses
superimppics = 0; % make graphs of impulse responses but superimposed for each variable one graph
threeDpics = 0;     % make funky 3D graphs of impulses evaluated over entire range of interactions
diffpics = 0;   % make graphs of differences of impulses over different evaluations
cumulative = 1; % make graphs of cumulative impulse response functions
supercum = 0; % make graphs of cumulative impulse responses but superimposed for each variable one graph
cumthreeDpics = 0; % make funky 3D graphs of cumulative impulses evaluated over entire range of interactions
cumdiffpics = 0; % make graphs of differences of cumulative impulses over different evaluations
contourplots = 0; % make graphs of ZERO-CONTOUR of impulses over all choices of interactions (shows when the impulse first crosses the zero line as a
% function of the debt ratio); Need threeDpics = 1 for this!!!
closepics       = 0;    % close figures again after saving such that not so many windows are open
%------------------------------------
% Define Regressors
%------------------------------------
datahelp=[log(rgovcons) log(realgdp) log(rprivinv) log(debtratio./100) netexp./realgdp];
dataraw=[rgovcons realgdp rprivinv debtratio/.100 netexp./realgdp];
varnames={'G' 'Y' 'I' 'D' 'TB'};
% compute share of respective variable of GDP:
var1_Y = nanmean(dataraw(:,1)./realgdp);
var2_Y = nanmean(dataraw(:,2)./realgdp);
var3_Y = nanmean(dataraw(:,3)./realgdp);
var4_Y = 1;
var5_Y = 1;
gdpfractions = [var1_Y var2_Y var3_Y var4_Y var5_Y];

%---------------------------------------
% Setting Restrictions
%---------------------------------------
% 5 eq. 1 int.
restr = [0 0 0 0 0 0 0 0 0 0
    0 0 0 0 0 0 0 0 0 0
    0 0 0 0 0 0 0 0 0 0
    0 0 0 0 0 0 0 0 0 0
    0 0 0 0 0 0 0 0 0 0];

%-----------------------------------
%-----------------------------------
%-------- AUTOMATIC PART -----------
%-----------------------------------
%-----------------------------------


%-----------------------------------
% Demeaning the Data
%-----------------------------------
if demean>-1
    data=paneldetrend(datahelp,I,demean);
    Idata=paneldetrend(Idata,I,demean);
else
    data = datahelp;
    Idata = Idata;
end
%---------------------------------------
% Defining values at which interactions are to be evaluated for the IRF
% rows correspond to the number of interaction terms, that you included;
% columns correspond to the different interaction values that you want to
% be included.
%---------------------------------------
nint=size(Idata,2);   % number of interaction terms
nvar=size(data,2);   % number of ebndogenous variables
values=[];
origvalues=[];
percentiles=5:95;
% This has entire distribution in 5% steps of each interaction variable in
% rows. Need to pick from rows and stack to get interesting combinations
for i=1:nint
    values=[values;prctile(Idata(:,i),percentiles)];
    origvalues = [origvalues;prctile(origIdata(:,i),percentiles)];
end
% All values
evalsteps = [10 30 50 70 90];

valuesall = zeros(nint,size(values,2),nint);
origvaluesall = zeros(nint,size(origvalues,2),nint);

% each page of valuesall should be used for one graph as it shows a
% different set of interaction scenarios
for i=1:nint
    valuesall(i,:,i) = values(i,:);
    origvaluesall(i,:,i) = origvalues(i,:);
end

%---------------------------------------
%---------------------------------------
% ESTIMATION
%---------------------------------------
%---------------------------------------

% If restrictions are made on LHS but not RHS variables this allows the use
% of seperate datasets by just creating missing observations in the LHS
% otherwise: xdata=ydata=data

ydata=data;
xdata=data;
%---------------------------------------BETAMAT
% Estimation of Structural Form in Bayesian fashion
%---------------------------------------
[BETAMAT,STERRMAT,ERRORMAT,positioner,responses]=interactPVARb(ydata,xdata,I,lag,Idata,nexog,restr,timing,valuesall,period,nrep,nburn,gdpfractions,percentgdpIRF,shock);

toc % stop timer

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                           GRAPHICS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
cumvarplot = [1 2 3 5]; % does not make sense to plot cumulative IRFs for all variables
gengraphs

