function [BETA,beta_stdevs,errors,R,responses]= bayesianest(X,Y,Idata_adj,nvar,nint,lag,values,period,nrep,nburn,gdpfractions,percentgdpIRF,shockto)

%=========================================================================
% Some preliminary stuff:
% ----------------------------
it_print = 100;

BETA = [];
ERRCOV = [];
R = zeros(nvar,nvar);
T = size(X,1);
errors = [];
edmat = [];
betavec = [];
sigmavec = [];
%=========================================================================
% Get OLS estimates first (can be used to center prior over, too):
% ----------------------------
Xcont{1}=X;
for k =1:nvar
    res(k).beta          = (Xcont{k}'*Xcont{k})^(-1)*Xcont{k}'*Y(k).Y;
    error(k).ols         = Y(k).Y-Xcont{k}*res(k).beta;
    errorunres(k).ols    = error(k).ols;
    variance(k).ols = (error(k).ols'*error(k).ols)/(size(Y(k).Y,1)-(size(find(res(k).beta),1))); % Variance of regression error
    errorvariance{k} = (error(k).ols'*error(k).ols)/(size(Y(k).Y,1)-(size(find(res(k).beta),1))); % Variance of regression error in one vector
    R(k,k) = variance(k).ols;
    std(k).error    = (diag((Xcont{k}'*Xcont{k})^(-1))*variance(k).ols).^0.5;
    betavar{k} = diag((Xcont{k}'*Xcont{k})^(-1))*variance(k).ols;
    Yh = Idata_adj(:,1+(k-1)*(nint+1):k*(nint+1));
    Xcont{k+1} = [Yh  Xcont{k}];
end
% OLS parameters in vector form:
a_OLS = [];
for i = 1:nvar
    a_OLS = [a_OLS;res(i).beta];
end
%%%%%%%%%%%%%%%%%%%%%%%%%%
for i=1:nvar
    YY(:,i)=Y(i).Y;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%
% Define a matrix that will hold all entries of the beta draws in one matrix
% as opposed to cells:
betafinaldraw=zeros(size(res(nvar).beta,1),nvar);
varbetamat=zeros(size(std(nvar).error,1),nvar);
betaevalb = zeros(nvar*(lag+1),nvar,size(values,2),nrep);
Pkomega =zeros(nvar,nvar,size(values,2));
azero =zeros(nvar,nvar,size(values,2));
Bv = zeros(nvar,nvar,lag);
responses = zeros(nvar,nvar,period);
for iter=1:nvar
    eval(['policyresponse' num2str(iter) '= zeros(period,size(values,2),nrep);']) % This should be avoided and changed
end
%=========================================================================
% Specify priors first:
% ----------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%% Informative based on OLS %%%%%%%%%%%%%%%%%%%
% beta_i ~ N(beta_OLS,4I), where k = 1,...,nvar
for k=1:nvar
    beta_prmean{k} = res(k).beta;
    inv_beta_prvar{k} = inv(10*diag(betavar{k}));
end

% Prior on covariance SIGMA_i ~ iG(a, b), where k = 1,...,N
v0 = .01*T;
v1 = (v0+T);
s02 = errorvariance;
for k=1:nvar
    v0s02{k}=v0*s02{k};
end
%=========================================================================
%  Sample beta and Sigma
% ----------------------------
for irep = 1:nrep + nburn    %  GIBBS iterations starts here
    % Print iterations
    if mod(irep,it_print) == 0
        disp(irep); toc;
    end
    % Since the covariance matrix of the error (SIGMA) in this equation is
    % diagonal, we can estimate the parameters equation-by-equation
    for k =1:nvar
        
        beta_postvar{k} = inv(inv_beta_prvar{k} + inv(R(k,k))*Xcont{k}'*Xcont{k});
        beta_postmean{k} = beta_postvar{k}*(inv_beta_prvar{k}*beta_prmean{k} + inv(R(k,k))*Xcont{k}'*Y(k).Y);
        betadraw{k}= beta_postmean{k}' + randn(1,size(beta_prmean{k},1))*chol(beta_postvar{k});
        % Now place cell entries into one single matrix
        betafinaldraw(size(res(nvar).beta,1)-size(res(k).beta,1)+1:end,k)=betadraw{k};
        % or vector
        betavec = [betavec betadraw{k}];
        
        ed = Y(k).Y - Xcont{k}*betadraw{k}'; % errors
        
        % Sample SIGMA(k,k) from iGamma
        v1s12 = v0s02{k} + (ed'*ed);
        Ridraw = inv(gamrnd(.5*v1,(.5*v1s12)^(-1))); % inverted to get variance as opposed to precision
        R(k,k) = Ridraw;
        sigmavec = [sigmavec Ridraw];
        
        edmat = [edmat ed];
        variance_err = ed'*ed/(size(Y(k).Y,1)-nvar*lag-1);
        stdev_beta{k} = (diag((Xcont{k}'*Xcont{k})^(-1))*variance_err).^0.5; % Variance of beta
        % Now place cell entries into one single matrix
        varbetamat(size(std(nvar).error,1)-size(std(k).error,1)+1:end,k)=stdev_beta{k};
    end
    %  Save after burn-in draws
    % ----------------------------
    %---------------------------------------
    % Create IRF
    %---------------------------------------
    if irep > nburn
        for ii=1:size(values,3) % over scenarios, ie.e holding different variables constant and varying one for example
            for i=1:size(values,2) % over columns in values, i.e. values within one scenario
                betaevalb(:,:,i,irep-nburn)=evalpvar(betafinaldraw,values(:,i,ii),lag);
                [betairf(:,:) Pkomega(:,:,i) azero(:,:,i)]=irfpreppvar(betaevalb(:,:,i,irep-nburn),R,lag);
                for jj = 1:lag
                    Bv(1:nvar,1:nvar,jj) = betairf((jj-1)*nvar+1:jj*nvar,:)';
                end
                shock = Pkomega(:,:,i)';
                d = diag(diag(shock));
                shock = d\shock;
                if percentgdpIRF == 1
                    responses=impulse(Bv,shock,period);
                    %     express each response in terms of percentage
                    %     points of gdp (not for variables which are expressed in GDP shares of course).
                    responses(1,:,:)=responses(1,:,:)*gdpfractions(1);
                    responses(2,:,:)=responses(2,:,:)*gdpfractions(2);
                    responses(3,:,:)=responses(3,:,:)*gdpfractions(3);
                    responses(4,:,:)=responses(4,:,:)*gdpfractions(4);
                    responses(5,:,:)=responses(5,:,:)*gdpfractions(5);
                    
                    %     normalize such that shock to variable is one percentage point of gdp.
                    responses(:,1,:)=responses(:,1,:)/responses(1,1,1);
                    responses(:,2,:)=responses(:,2,:)/responses(2,2,1);
                    responses(:,3,:)=responses(:,3,:)/responses(3,3,1);
                    responses(:,4,:)=responses(:,4,:)/responses(4,4,1);
                    responses(:,5,:)=responses(:,5,:)/responses(5,5,1);
                    
                    % Restrict to policy shocks and split up into different
                    % arrays - one for each responding variable
                    for iter=1:nvar
                        eval(['policyresponse' num2str(iter) '(:,i,irep-nburn)=squeeze(responses(iter,shockto,:,:,:,:));']) % This should be avoided and changed
                    end
                    
                else
                    responses(:,:,:,i,ii,irep-nburn)=impulse(Bv,shock,period);
                end
            end
        end
        %after discarding burnin, store all draws
        betavector(:,irep-nburn) = betavec';
        sigmavector(:,irep-nburn) = sigmavec';
        BETA(:,:,irep-nburn) = betafinaldraw;                % beta draws
        ERRCOV(:,:,irep-nburn) = R;                            % error variance draws
        errors(:,:,irep-nburn) = edmat; % structural errors
        beta_stdevs(:,:,irep-nburn) = varbetamat;
    end
    edmat = [];
    betavec = [];
    sigmavec = [];
end % END main Gibbs loop

% stack responses back into one big array
responses=[];
for iter=1:nvar
    eval(['responses(iter).responses = policyresponse' num2str(iter) ';']); % This should be avoided and changed
end
