%
%function evalbeta=evalpvar(beta,values,lag)
%
%===================================================================
% Evaluates the beta ceofficients that are the output of the
% interactPVAR.m command at the prespecified values for the inter-
% action terms.
% Required inputs are
% - beta:   output of the interactpvar (m-by-n)
% - values: prespecified values for the interaction terms. Matrix of
%           dimension: (# of evaluations - by- # of interactions)
% - lag:    # of lags in the estimation
%===================================================================
%
% See also irfpreppvar, interactPVAR
%
% Written by Pascal Towbin and Sebastian Weber - 20.02.2009

function evalbeta=evalpvar(beta,values,lag)

nint=size(values,1);	 % number of interaction terms
neq=size(beta,2);		 % number of equations
values=[1
    values];

% Evaluates the betas at the wished values

% These are the contemporaneous paramters
betaeval(1).b=beta(1:((1+nint)*(neq-1)),:);
% These are the lagged parameters
for l=1:lag
    betaeval(l+1).b=beta(1+(1+nint)*(neq-1)+((1+nint)*neq)*(l-1):(1+nint)*(neq-1)+((1+nint)*neq)*l,:);
end
% ------------------
% Now add up each slope parameter with its slope drifter for each endogenous parameter (e.g. alpha1 +
% alpha2*XInt etc.)
% For the contemporaneous parameters:
for k=1:neq
    for i=1:neq-1
        betaeval2(1).b(i,k)=betaeval(1).b(1+(1+nint)*(i-1):(1+nint)*i,k)' * values;
    end
end
%For the lagged parameters:
for l=1:lag
    for k=1:neq
        for i=1:neq
            betaeval2(l+1).b(i,k)=betaeval(l+1).b(1+(1+nint)*(i-1):(1+nint)*i,k)' * values;
        end
    end
end
% ---------------------------
% Now stack all interacted terms below each other into one matrix 
evalbeta=[];
for l=1:lag+1
    evalbeta=[evalbeta
              betaeval2(l).b];
end

for i=1:neq
    constanteval(1,i)=beta(size(beta,1)-nint:size(beta,1),i)'*values;
end
evalbeta=[evalbeta
          constanteval];
