qus = [.05,.1,.16,.5,.84,.90,.95]; % quantiles for error bands and median

colors = lines(size(evalsteps,2));
origresponses=responses;
shading3 = .5*[1 1 1];
transp = .5;

% do manipulations of original responses and save in separate structures to get things like cumulative
% responses
for i=1:nvar
    responses(i).quantiles = quantile(responses(i).responses,qus,3);
    responses(i).evalsteps = responses(i).responses(:,evalsteps,:);
    responses(i).cumresponses = cumsum(responses(i).responses(:,:,:),1);
    responses(i).cumquantiles = quantile(responses(i).cumresponses(:,:,:),qus,3);
    responses(i).cumevalsteps = responses(i).cumresponses(:,evalsteps,:);
end

%------------------------------------------
% GENERATING PICTURES II
%------------------------------------------
if imppics==1
    totallines = size(datahelp,2);
    cols = size(evalsteps,2);
    xpoints=[1:period,fliplr(1:period)];
    figure('Name','Impulse response functions for various interaction values');
    line = 0;
    for numpic=1:nvar
        line=line+1;
        ymin = min(min(min(responses(line).quantiles(:,evalsteps,:)))); % set max and min y axis values for each row of the graph
        ymax = max(max(max(responses(line).quantiles(:,evalsteps,:))));
        
        for i=1:cols
            filled3=[squeeze(responses(line).quantiles(:,evalsteps(i),1))',... % same for wider bands
                fliplr(squeeze(responses(line).quantiles(:,evalsteps(i),7))')];
            
            subplot(totallines,cols,(cols)*(line-1)+i)
            
            fill(xpoints,filled3,shading3,'EdgeColor',shading3,'EdgeAlpha',transp);%plot the shading between two outer bands
            alpha(transp)
            hold on
            
            plot(1:period,squeeze(responses(line).quantiles(:,evalsteps(i),4)),'LineStyle','-','Color','k');
            hold on
            plot(zeros(1,period),'LineStyle',':','Color','k')   % plot dotted line at zero
            axis([1 period ymin ymax]);
            box on
            if i==1;
                ylabel([num2str(varnames{numpic})])
            end
            if line==1;
                title(['D/Y: ' num2str(round(origvaluesall(evalsteps(i)))) '%'])
            end
            set(gca,'XTickLabel',0:5:period-1,'XTick',1:5:period,'Color','w')
        end
    end
    set(gcf, 'Color', 'w');
    set(gcf, 'Position', get(0,'Screensize')); % Maximize figure
    if closepics == 1
        close all
    end
end
% %------------------------------------------
% % GENERATING PICTURES - 3-D GRAPHS for each variable responding to defined shock for all
% % evalutions of the interaction terms
% %------------------------------------------
if threeDpics==1
    % for 3D plots
    picnames = varnames;
    if contourplots == 1
        % for contourplots
        picnames1 = varnames;
    end
    %     for ii=1:size(responses,5)
    for var=1:nvar
        % Now do 3-D graphs for each variable responding to defined shock for all
        % evalutions of the interaction terms
        figure('Name',['3-D graph for variable ' num2str(var)],'Color','w')
        allimps = squeeze(responses(var).quantiles(:,:,4));
        surf(allimps,'LineStyle','-','EdgeColor','black','FaceColor','none') %%%%%%% B&W picture %%%%%%%%%%
        hold on
        
        surf(zeros(period,size(values,2)),'LineStyle','none','FaceColor',[0.5 0.5 0.5],'FaceAlpha',0.8) % The zero surface
        
        view([150,15])
        xlabel('D/Y evaluation level in %','FontWeight','bold','FontSize',20)
        ylabel('Periods after shock','FontWeight','bold','FontSize',20,'Rotation',-20)
        zlabel('Response in % of GDP','FontWeight','bold','FontSize',20)
        title(num2str(varnames{var}),'FontWeight','bold','FontSize',22)
        set(gca,'XTickLabel',round(origvalues(1:10:size(origvalues,2))),'XTick',[1:10:size(origvalues,2)],'XDir','reverse','YDir','reverse','Color','w','FontWeight','bold','FontSize',12)
        set(gca,'YTickLabel',0:5:period-1,'YTick',1:5:period,'Color','w','FontWeight','bold','FontSize',12)
        orient rotated;
        set(gcf, 'Position', get(0,'Screensize')); % Maximize figure
        if closepics == 1
            close
        end
        if contourplots == 1
            % %------------------------------------------
            % % GENERATING PICTURES - ZERO-CONTOUR of impulses over all choices of
            % interactions (shows when the impulse first crosses the zero line as a
            % function of the debt ratio)
            % %------------------------------------------
            figure('Name',['Zero-Contour response for variable ' num2str(var)],'Color','w')
            colormap([.5 .5 .5]);
            contourf(allimps,[0 0],'LineWidth',2,'LineColor','k')
            xlabel('D/Y evaluation level in %','FontWeight','bold','FontSize',20)
            ylabel('Periods after shock','FontWeight','bold','FontSize',20)
            title(['Zero-contour of ' num2str(varnames{var})],'FontWeight','bold','FontSize',20)
            set(gca,'XTickLabel',round(origvalues(1:10:size(origvalues,2))),'XTick',[1:10:size(origvalues,2)],'Color','w','FontWeight','bold','FontSize',12)
            set(gca,'YTickLabel',0:5:period-1,'YTick',1:5:period,'Color','w','FontWeight','bold','FontSize',12)
            set(gcf, 'Position', get(0,'Screensize')); % Maximize figure
            if closepics == 1
                close all
            end
        end
    end
end

% %------------------------------------------
% % GENERATING PICTURES - DIFFERENCES of impulses over selceted choices of
% interactions
% %------------------------------------------
%This part here does differencing first and then quantiles
if diffpics==1
    numdiffpics = 2; % so many differences to build (have to do it manually in loop below)
    DIFFMAT1 = zeros(nvar,period,numdiffpics,nrep);
    for ndiffs=1:numdiffpics
        for var=1:nvar
            DIFFMAT1(var,:,1,:) = responses(var).evalsteps(:,5,:) - responses(var).evalsteps(:,1,:);
            DIFFMAT1(var,:,2,:) = responses(var).evalsteps(:,4,:) - responses(var).evalsteps(:,2,:);
        end
    end
    DIFFMAT1 = quantile(DIFFMAT1,qus,4);
    figure('Name','Differences between interaction choices from most extreme (left) to less extreme (right) coulmns and variables in rows to specified shock (IRFs)','PaperOrientation','landscape','Color','w');
    for var=1:nvar
        ymin = min(min(min(DIFFMAT1(var,:,:,:)))); % set max and min y axis values for each row of the graph
        ymax = max(max(max(DIFFMAT1(var,:,:,:))));
        for ndiffs=1:numdiffpics
            filled3=[squeeze(DIFFMAT1(var,:,ndiffs,1)),... % same for wider bands
                fliplr(squeeze(DIFFMAT1(var,:,ndiffs,7)))];
            
            subplot(nvar,numdiffpics,numdiffpics*(var-1)+ndiffs)
            
            fill(xpoints,filled3,shading3,'EdgeColor',shading3,'EdgeAlpha',transp);%plot the shading between two outer bands
            alpha(transp)
            hold on
            
            plot(squeeze(DIFFMAT1(var,:,ndiffs,4)),'LineStyle','-','Color','k'); % median only plotted
            hold on
            plot(zeros(1,period),'LineStyle',':','Color','k')   % plot dotted line at zero
            axis([1 period ymin ymax]);
            if var==1;
                if ndiffs==1
                    title(['resp. at D/Y=' num2str(round(origvaluesall(evalsteps(end)))) '% minus resp. at D/Y=' num2str(round(origvaluesall(evalsteps(1)))) '%'])
                end
                if ndiffs==2
                    title(['resp. at D/Y=' num2str(round(origvaluesall(evalsteps(end-1)))) '% minus resp. at D/Y=' num2str(round(origvaluesall(evalsteps(2)))) '%'])
                end
                if ndiffs==3
                    title(['3rd last minus 3rd'])
                end
                if ndiffs==4
                    title(['4th last minus 4th'])
                end
            end
            if ndiffs==1;
                ylabel([num2str(varnames{var})])
            end
            set(gca,'XTickLabel',0:5:period-1,'XTick',1:5:period,'Color','w')
        end
    end
    orient rotated;
    set(gcf, 'Color', 'w');
    set(gcf, 'Position', get(0,'Screensize')); % Maximize figure
    if closepics == 1
        close all
    end
end

%------------------------------------------
% GENERATING PICTURES (cumulative IRFs)
%------------------------------------------
% responses are neq*nshocks*periods*evaluations*draws
% responses(:,:,:,i,irep-nburn)
% For cumulative responses it only makes sense to use flow variables to
% give a sense of present discounted values (it makes sense for flow
% variables, though perhaps some discounting could be applied)
if cumulative==1
    totallines = size(cumvarplot,2);
    cols = size(evalsteps,2);
    xpoints=[1:period,fliplr(1:period)];
    line = 0;
    figure('Name','Cumulative IRFs at different interaction choices','PaperOrientation','landscape');
    for numpic=1:totallines
        line=cumvarplot(numpic);
        ymin = min(min(min(responses(line).cumquantiles(:,evalsteps,:)))); % set max and min y axis values for each row of the graph
        ymax = max(max(max(responses(line).cumquantiles(:,evalsteps,:))));
        for i=1:cols
            filled3=[squeeze(responses(line).cumquantiles(:,evalsteps(i),1))',... % same for wider bands
                fliplr(squeeze(responses(line).cumquantiles(:,evalsteps(i),7))')];
            
            subplot(totallines,cols,(cols)*(numpic-1)+i)
            
            fill(xpoints,filled3,shading3,'EdgeColor',shading3,'EdgeAlpha',transp);%plot the shading between two outer bands
            alpha(transp)
            hold on
            
            plot(1:period,squeeze(responses(line).cumquantiles(:,evalsteps(i),4)),'LineStyle','-','Color','k');
            hold on
            plot(zeros(1,period),'LineStyle',':','Color','k')   % plot dotted line at zero
            axis([1 period ymin ymax]);
            if i==1;
                ylabel([num2str(varnames{line})])
            end
            if line==1;
                title(['D/Y: ' num2str(round(origvaluesall(evalsteps(i)))) '%'])
            end
            %             set(gca,'FontSize',16,'FontWeight','bold')
            set(gca,'XTickLabel',0:5:period-1,'XTick',1:5:period,'Color','w')
        end
    end
    orient rotated;
    set(gcf, 'Color', 'w');
    set(gcf, 'Position', get(0,'Screensize')); % Maximize figure
    set(gca,'XTickLabel',0:5:period-1,'XTick',1:5:period,'Color','w')
    if closepics == 1
        close all
    end
end

% %------------------------------------------
% % GENERATING PICTURES - 3-D GRAPHS of cumulative IRFs for each variable responding to defined shock for all
% % evalutions of the intereaction terms
% %------------------------------------------
if cumthreeDpics==1
    %for 3D graphs
    picnames = varnames;
    if contourplots ==1
        % for contour plots:
        picnames1 = varnames;
    end
    %     for ii=1:size(responses,5)
    for var=cumvarplot;
        % Now do 3-D graphs for each variable responding to defined shock for all
        % evalutions of the interaction terms
        figure('Name',['3-D cum IRF graph for variable ' num2str(var)],'Color','w')
        allimps1 = squeeze(responses(var).cumquantiles(:,:,4));
        surf(allimps1,'LineStyle','-','EdgeColor','black','FaceColor','none') %%%%%%% B&W picture %%%%%%%%%%
        hold on
        
        surf(zeros(period,size(values,2)),'LineStyle','none','FaceColor',[0.5 0.5 0.5],'FaceAlpha',0.8) % The zero surface
        
        view([150,15])
        xlabel('D/Y evaluation level in %','FontWeight','bold','FontSize',20)
        ylabel('Periods after shock','Rotation',-20,'FontWeight','bold','FontSize',20)
        zlabel('Response in % of GDP','FontWeight','bold','FontSize',20)
        title(num2str(varnames{var}),'FontWeight','bold','FontSize',22)
        set(gca,'XTickLabel',round(origvalues(1:10:size(origvalues,2))),'XTick',[1:10:size(origvalues,2)],'XDir','reverse','YDir','reverse','Color','w','FontWeight','bold','FontSize',12)
        set(gcf, 'Position', get(0,'Screensize')); % Maximize figure
        set(gca,'YTickLabel',0:5:period-1,'YTick',1:5:period,'Color','w','FontWeight','bold','FontSize',12)
        orient rotated;
        if closepics == 1
            close
        end
        if contourplots == 1
            % %------------------------------------------
            % % GENERATING PICTURES - ZERO-CONTOUR of CUMULATIVE impulses over all choices of
            % interactions (shows when the impulse first crosses the zero line as a
            % function of the debt ratio)
            % %------------------------------------------
            figure('Name',['Zero-Contour response for variable ' num2str(var)],'Color','w')
            colormap([.5 .5 .5]);
            hold all
            trick=contourf(allimps1,[0 0],'LineColor','k');
            xlabel('D/Y evaluation level in %','FontWeight','bold','FontSize',20)
            ylabel('Periods after shock','FontWeight','bold','FontSize',20)
            title(['Zero-contour of ' num2str(varnames{var})],'FontWeight','bold','FontSize',20)
            set(gca,'XTickLabel',round(origvalues(1:10:size(origvalues,2))),'XTick',[1:10:size(origvalues,2)],'Color','w','FontWeight','bold','FontSize',12)
            set(gcf, 'Position', get(0,'Screensize')); % Maximize figure
            set(gca,'YTickLabel',0:5:period-1,'YTick',1:5:period,'Color','w','FontWeight','bold','FontSize',12)
            if closepics == 1
                close all
            end
        end
    end
    %     end
end

% %------------------------------------------
% % GENERATING PICTURES - DIFFERENCES of CUMULATIVE impulses over selceted choices of
% interactions
% %------------------------------------------
%This part here does differencing first and then quantiles
if cumdiffpics==1
    numdiffpics = 2; % so many difference pairs to build (have to do it manually in loop below)
    DIFFMAT1 = zeros(nvar,period,numdiffpics,nrep);
    for ndiffs=1:numdiffpics
        for var=1:nvar
            DIFFMAT1(var,:,1,:) = responses(var).cumevalsteps(:,5,:) - responses(var).cumevalsteps(:,1,:);
            DIFFMAT1(var,:,2,:) = responses(var).cumevalsteps(:,4,:) - responses(var).cumevalsteps(:,2,:);
        end
    end
    DIFFMAT1 = quantile(DIFFMAT1,qus,4);
    figure('Name','Differences between interaction choices from most extreme (left) to less extreme (right) coulmns and variables in rows to specified shock (cum. IRFs)','PaperOrientation','landscape','Color','w');
    line = 0;
    for var=cumvarplot
        line = line + 1;
        ymin = min(min(min(DIFFMAT1(var,:,:,:)))); % set max and min y axis values for each row of the graph
        ymax = max(max(max(DIFFMAT1(var,:,:,:))));
        for ndiffs=1:numdiffpics
            filled3=[squeeze(DIFFMAT1(var,:,ndiffs,1)),... % same for wider bands
                fliplr(squeeze(DIFFMAT1(var,:,ndiffs,7)))];
            
            subplot(size(cumvarplot,2),numdiffpics,numdiffpics*(line-1)+ndiffs)
            fill(xpoints,filled3,shading3,'EdgeColor',shading3,'EdgeAlpha',transp);%plot the shading between two outer bands
            alpha(transp)
            hold on
            
            
            plot(squeeze(DIFFMAT1(var,:,ndiffs,4)),'LineStyle','-','Color','k'); % median only plotted
            hold on
            plot(zeros(1,period),'LineStyle',':','Color','k')   % plot dotted line at zero
            axis([1 period ymin ymax]);
            if var==1;
                if ndiffs==1
                    title(['resp. at D/Y=' num2str(round(origvaluesall(evalsteps(end)))) '% minus resp. at D/Y=' num2str(round(origvaluesall(evalsteps(1)))) '%'])
                end
                if ndiffs==2
                    title(['resp. at D/Y=' num2str(round(origvaluesall(evalsteps(end-1)))) '% minus resp. at D/Y=' num2str(round(origvaluesall(evalsteps(2)))) '%'])
                end
                if ndiffs==3
                    title(['3rd last minus 3rd'])
                end
                if ndiffs==4
                    title(['4th last minus 4th'])
                end
            end
            if ndiffs==1;
                ylabel([num2str(varnames{var})])
            end
            set(gca,'XTickLabel',0:5:period-1,'XTick',1:5:period,'Color','w')
        end
    end
    orient rotated;
    set(gcf, 'Color', 'w');
    set(gcf, 'Position', get(0,'Screensize')); % Maximize figure
    if closepics == 1
        close all
    end
end
% %------------------------------------------
% % SUPERIMPOSED GRAPHS
% %------------------------------------------

markers = {'o','s','d','p','>','^','*','.','x','v','<','h'};
if superimppics==1
    picnames = varnames;
    totallines = size(datahelp,2);
    cols = size(evalsteps,2);
    transp = .5;
    hleg = {};
    for ii=1:size(responses,5)
        line = 0;
        for numpic=1:nvar
            figure('Name','Impulse responses functions for various interaction values superimposed','PaperOrientation','landscape','Color','w');
            line=line+1;
            ymin = min(min(min(responses(line).quantiles(:,evalsteps,:)))); % set max and min y axis values for each row of the graph
            ymax = max(max(max(responses(line).quantiles(:,evalsteps,:))));
            for i=1:cols
                plot(1:period,squeeze(responses(line).quantiles(:,evalsteps(i),4)),'color','k','LineWidth',2,'Marker',markers{mod(i,numel(markers))},'MarkerSize',15); % plot the actual IRF
                hleg{i} = [num2str(round(origvaluesall(evalsteps(i)))) '% D/Y'];
                hold on
                axis([1 period ymin ymax]);
            end
            legend(hleg);
            plot(zeros(1,period),'LineStyle',':','Color','k')   % plot dotted line at zero
            box on
            set(gca,'FontSize',18,'FontWeight','bold');
            set(gcf, 'Position', get(0,'Screensize')); % Maximize figure
            set(gca,'XTickLabel',0:5:period-1,'XTick',1:5:period,'Color','w')
            title(num2str(varnames{numpic}),'FontSize',22,'FontWeight','bold')
            xlabel('periods after shock','FontSize',18,'FontWeight','bold');
            ylabel('Response in % of GDP','FontSize',22,'FontWeight','bold');
            if closepics == 1
                close all
            end
        end
    end
end

%------------------------------------------
% GENERATING PICTURES (cumulative IRFs)
%------------------------------------------
% responses are neq*nshocks*periods*evaluations*draws
% responses(:,:,:,i,irep-nburn)
if supercum == 1
    picnames = varnames;
    totallines = size(datahelp,2);
    cols = size(evalsteps,2);
    line = 0;
    transp = .5;
    for numpic=1:nvar
        line=line+1;
        ymin = min(min(min(responses(line).cumquantiles(:,evalsteps,:)))); % set max and min y axis values for each row of the graph
        ymax = max(max(max(responses(line).cumquantiles(:,evalsteps,:))));
        figure('Name','Cumulative IRFs at different interaction choices superimposed','PaperOrientation','landscape','Color','w');
        for i=1:cols
            plot(1:period,squeeze(responses(line).cumquantiles(:,evalsteps(i),4)),'color','k','LineWidth',2,'Marker',markers{mod(i,numel(markers))},'MarkerSize',15); % plot the actual IRF
            hleg{i} = [num2str(round(origvaluesall(evalsteps(i)))) '% D/Y'];
            hold on
            axis([1 period ymin ymax]);
        end
        legend(hleg);
        plot(zeros(1,period),'LineStyle',':','Color','k')   % plot dotted line at zero
        box on
        set(gca,'FontSize',16,'FontWeight','bold');
        set(gcf, 'Position', get(0,'Screensize')); % Maximize figure
        set(gca,'XTickLabel',0:5:period-1,'XTick',1:5:period,'Color','w')
        title(num2str(varnames{numpic}),'FontSize',22,'FontWeight','bold')
        xlabel('periods after shock','FontSize',18,'FontWeight','bold');
        ylabel('Response in % of GDP','FontSize',22,'FontWeight','bold');
        orient rotated;
        if closepics == 1
            close all
        end
    end
end
