%
%function [output] =  paneldetrend(data,I, power)
%
% ==========================================================================
% Allows to detrend panel data, according to three different methods defined
% by the input
% - Power:  =0 is demeaning
%           =1 is linear detrending
%           =2 is quadratic detrending
% - I:    Is the cross sectional dimension
% - data: Is the panel data with dimension (time*I - by - variable)
% ==========================================================================
% See also paneldiff, panellag
%
% Written by Pascal Towbin and Sebastian Weber, 20.02.2009
%

function [output] = paneldetrend(data,I,power)


[T K]=size(data);
time = size(data,1)/I;
if time ~= floor(time);
    display('Error: The time dimension is not identical across individuals!!')
end


%output=zeros(T,K)./zeros(T,K);
for actual_country=1:I
    output1=[];
    start=1+(actual_country-1)*time;
    ende=actual_country*time;
    
    
    trend=(0:time)';
    %%%%%Create regressors
    X=[];
    for i=0:power
        X=[X trend.^i];
    end
    
    %a=ifscode(start:ende)
    for var=1:K
        Y=data(start:ende,var);
        [ROWIND COLIND] = find(isfinite(Y));
        [ROWIND2 COLIND2] = find(isnan(Y));
        newY=zeros(size(ROWIND,1),1);
        newX=zeros(size(ROWIND,1),size(X,2));
        for k=1:size(ROWIND,1);
            newY(k,:)=Y(ROWIND(k,1),:);
            newX(k,:)=X(ROWIND(k,1),:);
        end;  %Remove NaN for estimations
       
        BETAA=(newX'*newX)^-1*newX'*newY;
        detrend_raw=newY-newX*BETAA;
        detrended=detrend_raw;
        detrend_new=detrended;
        %refill Nan to assure uniform data length
        no_isfinite=size(ROWIND,1);
        no_nan=size(ROWIND2,1);
        for k=1:no_nan;
            detrend_new(ROWIND2(k,1)+1:no_isfinite+k,1)=detrended(ROWIND2(k,1):no_isfinite+k-1,1);
            detrend_new(ROWIND2(k,1),1)=nan;
            detrended=detrend_new;
        end
        output1=[output1 detrended];
        
    end;
    output(start:ende,:)=output1;
end
end

%=======================================================================